/*
  TurboC, a library for porting Borland Turbo C to GNU gcc.
  Copyright 2002 Ronald S. Burkey
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  
  Contact Ron Burkey at info@birdsproject.org.

  Filename:	getchTurbo.c
  Purpose:	A Turbo C conio function getch.
  Mod history:	01/31/02 RSB	Created.
  		03/02/02 RSB	Added actual code.
*/

#include <stdio.h>
#include "conio.h"
#include "fnkeys.h"

//-----------------------------------------------------------------------
// ncurses returns various special keys as codes in the range 0x100-0x1ff.
// Borland's getch function returns these as 2-code sequences:  
// 0, followed by some other code (from fnkeys.h).  The following table
// converts (where non-zero) from ncurses to the 2nd Borland code.

static const char n2g[] = {
  /* 0x100 */ 0, 0, N_DOWN, N_UP, N_LEFT, N_RIGHT, 0, 0,
  /* 0x108 */ 0, N_F1, N_F2, N_F3, N_F4, N_F5, N_F6, N_F7,
  /* 0x110 */ N_F8, N_F9, N_F10, 0, 0, S_F1, S_F2, S_F3,
  /* 0x118 */ S_F4, S_F5, S_F6, S_F7, S_F8, 0, 0, 0,
  /* 0x120 */ 0, 0, 0, 0, 0, 0, 0, 0,
  /* 0x128 */ 0, 0, 0, 0, 0, 0, 0, 0,
  /* 0x130 */ 0, 0, 0, 0, 0, 0, 0, 0,
  /* 0x138 */ 0, 0, 0, 0, 0, 0, 0, 0,
  /* 0x140 */ 0, 0, 0, 0, 0, 0, 0, 0,
  /* 0x148 */ 0, 0, N_DEL, N_INS, 0, 0, 0, 0,
  /* 0x150 */ 0, 0, N_PGDN, N_PGUP, 0, 0, 0, 0,
  /* 0x158 */ 0, 0, 0, 0, 0, 0, 0, 0,
  /* 0x160 */ 0, 0, 0, 0, 0, 0, 0, 0,
  /* 0x168 */ 0, 0, N_HOME, 0, 0, 0, 0, 0,
  /* 0x170 */ 0, 0, 0, 0, 0, 0, 0, 0,
  /* 0x178 */ 0, 0, 0, 0, 0, 0, 0, 0,
  /* 0x180 */ 0, N_END, 0, 0, 0, 0, 0, 0
};


//-----------------------------------------------------------------------
// A buffer, for ungetching.

#define MAX_UNGETCH 128
static int NumUngetch = 0;
static char UngetchBuffer[MAX_UNGETCH];
FILE *KeyLog = NULL;

//-----------------------------------------------------------------------
int
getchTurbo (void)
{
  int KeyCode;
  if (!ConioInitialized)
    textmode (LASTMODE);
  // First take care of any ungetched keystrokes, or buffered multi-code 
  // keystrokes like up/down arrows.
  if (NumUngetch)
    return (UngetchBuffer[--NumUngetch]);
  // No buffered codes.  Wait for a real keystroke. 
  for (;;)
    {
      while (kbhit () == 0);
      KeyCode = getchNcurses ();
      if (KeyLog != NULL)
	fwrite (&KeyCode, sizeof (KeyCode), 1, KeyLog);
      // Treat backspace key specially, because it doesn't follow the
      // pattern of being converted to a 2-code sequence.
      if (KeyCode == 0x107)
	KeyCode = C_H;
      // If it's a regular 7-bit ASCII keystroke, we can simply return it
      // right now.  Otherwise, we have to interpret it as a multi-code
      // keystroke and buffer some of it.
      if (KeyCode < 128)
	return (KeyCode);
      if (KeyCode >= 0x100)
	{
	  KeyCode -= 0x100;
	  if (KeyCode < sizeof (n2g) && n2g[KeyCode] != 0)
	    {
	      ungetchTurbo (n2g[KeyCode]);
	      return (0);
	    }
	}
    }
}

//----------------------------------------------------------------------
int
ungetchTurbo (int ch)
{
  if (NumUngetch >= MAX_UNGETCH)
    return (EOF);
  UngetchBuffer[NumUngetch++] = ch;
  return (ch);
}
